@@ -1,17 +1,21 @@
 /*
  * common.c
- * Copyright (C) 2008 KLab Inc.
+ * Copyright (C) 2008-2012 KLab Inc.
  */
 #include "makuosan.h"
 
 mopt moption;
-mfile *mftop[2] = {NULL,NULL};
-mfile *mfreeobj = NULL;
-mhost *members  = NULL;
-int loop_flag   = 1;
+mfile *mftop[2]  = {NULL,NULL};
+mfile *mfreeobj  = NULL;
+mhost *members   = NULL;
+int send_rate    = 0;
+int view_rate    = 0;
+time_t send_time = 0;
 struct timeval curtime;
 struct timeval lastpong;
 BF_KEY EncKey;
+volatile sig_atomic_t log_level = 0;
+volatile sig_atomic_t loop_flag = 1;
 
 char *yesno(int n)
 {
@@ -111,15 +115,36 @@ uint32_t getrid()
 
 int workend(mcomm *c)
 {
-  if(c){
-    if(c->working && !c->cpid){
-      c->working = 0;
-      if(moption.commpass && !c->authchk){
-        cprintf(0, c, "password: \x1b]E");
+  char *m;
+  if(!c){
+    return(0);
+  }
+  if(!c->working){
+    return(0);
+  }
+  if(c->cpid){
+    return(0);
+  }
+  if(moption.commpass && !c->authchk){
+    m = "password: \x1b]E";
+  }else{
+    if(c->logover && strcmp("dsync", c->parse[0][0])){
+      if(cprintf(0, c, "[error] Log lost: %d lines\n", c->logover) == 0){
+        c->logover = 0;
       }else{
-        cprintf(0,c,"> ");
+        c->logover--;
       }
     }
+    if(c->logfail){
+      m = "\n> ";
+    }else{
+      m = "> ";
+    }
+  }
+  if(cprintf(0, c, m) == 0){
+    c->working = 0;
+  }else{
+    c->logover--;
   }
   return(0);
 }
@@ -146,14 +171,18 @@ void mfdel(mfile *m)
   mfile *p;
   mfile *n;
   if(m){
-    if(p = (mfile *)m->prev)
+    if((p = (mfile *)m->prev)){
       p->next = m->next;
-    if(n = (mfile *)m->next)
+    }
+    if((n = (mfile *)m->next)){
       n->prev = m->prev;
-    if(mftop[0] == m)
+    }
+    if(mftop[0] == m){
       mftop[0] = n;
-    if(mftop[1] == m)
+    }
+    if(mftop[1] == m){
       mftop[1] = n;
+    }
     mfree(m);
   }
 }
@@ -161,7 +190,7 @@ void mfdel(mfile *m)
 mfile *mfnew()
 {
   mfile *m;
-  if(m = mfalloc()){
+  if((m = mfalloc())){
     memset(m, 0, sizeof(mfile));
     m->mdata.head.maddr  = moption.maddr.sin_addr.s_addr;
     m->mdata.head.mport  = moption.maddr.sin_port;
@@ -177,7 +206,7 @@ mfile *mfnew()
 mfile *mfadd(int n)
 {
   mfile *m;
-  if(m = mfnew()){
+  if((m = mfnew())){
     if(!mftop[n]){
       mftop[n] =m;
     }else{
@@ -194,7 +223,7 @@ mfile *mfadd(int n)
 mfile *mfins(int n)
 {
   mfile *m;
-  if(m = mfnew()){
+  if((m = mfnew())){
     if(mftop[n]){
       mftop[n]->prev = m;
       m->next = mftop[n];
@@ -266,10 +295,10 @@ void member_del(mhost *t)
     return;
   }
   lprintf(0, "%s: %s (%s)\n", __func__, inet_ntoa(t->ad), t->hostname);
-  if(p = (mhost *)t->prev){
+  if((p = (mhost *)t->prev)){
     p->next = t->next;
   }
-  if(n = (mhost *)t->next){
+  if((n = (mhost *)t->next)){
     n->prev = t->prev;
   }
   if(members == t){
@@ -287,7 +316,7 @@ void member_del_message(int err, mhost *t, char *mess)
     return;
   }
   for(i=0;i<MAKUO_HOSTSTATE_SIZE;i++){
-    if(m = t->mflist[i]){
+    if((m = t->mflist[i])){
       if(m->comm){
         if(m->comm->working){
           cprintf(0, m->comm, "error: %s: %s\n", mess, m->cmdline);
@@ -440,7 +469,7 @@ void seq_setmark(mfile *m, uint32_t l, uint32_t h)
     }
     mm = delmark(mm);
   }
-  if(mn->next = m->mark){
+  if((mn->next = m->mark)){
     m->mark->prev = mn;
   }
   m->mark = mn;
@@ -487,7 +516,7 @@ void dump_hoststate(mfile *m, char *func)
   mhost   *t;
   uint8_t *r;
   for(t=members;t;t=t->next){
-    if(r=get_hoststate(t,m)){
+    if((r=get_hoststate(t,m))){
       lprintf(9,"%s: %s from %s %s\n", func, strrstate(*r), t->hostname, m->fn);
     }
   }
@@ -520,7 +549,7 @@ uint8_t *get_hoststate(mhost *t, mfile *m)
 uint8_t *set_hoststate(mhost *t, mfile *m, uint8_t state)
 {
   uint8_t *s;
-  if(s = get_hoststate(t,m)){
+  if((s = get_hoststate(t,m))){
     *s = state;
   }
   return(s);
@@ -532,14 +561,14 @@ int ack_clear(mfile *m, int state)
   mhost   *t;
   for(t=members;t;t=t->next){
     if(!m->sendto){
-      if(s = get_hoststate(t, m)){
+      if((s = get_hoststate(t, m))){
         if(state == -1 || *s == state){
           *s = MAKUO_RECVSTATE_NONE;
         }
       }
     }else{
       if(!memcmp(&(m->addr.sin_addr), &(t->ad), sizeof(m->addr.sin_addr))){
-        if(s = get_hoststate(t, m)){
+        if((s = get_hoststate(t, m))){
           if(state == -1 || *s == state){
             *s = MAKUO_RECVSTATE_NONE;
             return(1);
@@ -736,8 +765,8 @@ int mremove(char *base, char *name)
     sprintf(path, "%s/%s", base, name);
   }
   if(is_dir(path)){
-    if(d = opendir(path)){
-      while(dent=readdir(d)){
+    if((d = opendir(path))){
+      while((dent=readdir(d))){
         if(!strcmp(dent->d_name, "."))
           continue;
         if(!strcmp(dent->d_name, ".."))
@@ -778,7 +807,7 @@ int mcreatedir(char *base, char *name, mode_t mode)
   while(p){
     strcat(path, "/");
     strcat(path, p);
-    if(p = strtok(NULL,"/")){
+    if((p = strtok(NULL,"/"))){
       if(!is_dir(path)){
         remove(path);
         if(mkdir(path,mode) == -1){
@@ -848,7 +877,7 @@ int space_escape(char *str)
 mfile *mkreq(mdata *data, struct sockaddr_in *addr, uint8_t state)
 {
   mfile *a;
-  if(a = mfins(MFSEND)){
+  if((a = mfins(MFSEND))){
     a->mdata.head.opcode = data->head.opcode;
     a->mdata.head.reqid  = data->head.reqid;
     a->mdata.head.seqno  = data->head.seqno;
@@ -871,7 +900,7 @@ mfile *mkack(mdata *data, struct sockaddr_in *addr, uint8_t state)
       return(a);
     }
   }
-  if(a = mfins(MFSEND)){
+  if((a = mfins(MFSEND))){
     a->mdata.head.flags |= MAKUO_FLAG_ACK;
     a->mdata.head.opcode = data->head.opcode;
     a->mdata.head.reqid  = data->head.reqid;
@@ -968,7 +997,8 @@ int data_safeset16(mdata *data, uint16_t val)
   if(data->head.szdata + sizeof(uint16_t) > MAKUO_BUFFER_SIZE){
     return(-1);
   }
-  *(uint16_t *)(data->data + data->head.szdata) = htons(val);
+  val = htons(val);
+  memcpy((void *)(data->data + data->head.szdata), (void *)(&val), sizeof(val));
   data->head.szdata += sizeof(uint16_t);
   return(0);
 }
@@ -978,7 +1008,8 @@ int data_safeset32(mdata *data, uint32_t val)
   if(data->head.szdata + sizeof(uint32_t) > MAKUO_BUFFER_SIZE){
     return(-1);
   }
-  *(uint32_t *)(data->data + data->head.szdata) = htonl(val);
+  val = htonl(val);
+  memcpy((void *)(data->data + data->head.szdata), (void *)(&val), sizeof(val));
   data->head.szdata += sizeof(uint32_t);
   return(0);
 }
