@@ -1,6 +1,6 @@
 /*
  * mexec.c
- * Copyright (C) 2008 KLab Inc.
+ * Copyright (C) 2008-2012 KLab Inc.
  */
 
 #include "makuosan.h"
@@ -22,6 +22,8 @@ char *command_list[]={"quit",
                       "help",
                       NULL};
 
+int mexec_scan_child(int fd, char *base, char *sendhost, int mode, mcomm *c, gid_t gid);
+
 mfile *mexec_with_dsync(mcomm *c, char *fn, int dryrun, int recurs, mhost *t)
 {
   mfile *m = mfadd(MFSEND);
@@ -150,31 +152,32 @@ int mexec_scan_dir(int fd, char *base, char *sendhost, int mode, mcomm *c, int b
 
   d = opendir(base);
   if(!d){
-    /* directory open error */
     mexec_scan_echo(fd, "directory open error %s", base);
-  }else{
-    while(dent=readdir(d)){
-      if(!loop_flag){
-        return(1);
-      }
-      if(!strcmp(dent->d_name, ".")){
-        continue;
-      }
-      if(!strcmp(dent->d_name, "..")){
-        continue;
-      }
-      if(baseflag){
-        sprintf(path, "%s/%s", base, dent->d_name);
-      }else{
-        strcpy(path, dent->d_name);
-      }
-      space_escape(path);
-      if(r = mexec_scan_child(fd, path, sendhost, mode, c, gid)){
-        return(r);
-      }
-    }
-    closedir(d);
+    return(0);
   }
+  while((dent=readdir(d))){
+    if(!loop_flag){
+      closedir(d);
+      return(1);
+    }
+    if(!strcmp(dent->d_name, ".")){
+      continue;
+    }
+    if(!strcmp(dent->d_name, "..")){
+      continue;
+    }
+    if(baseflag){
+      sprintf(path, "%s/%s", base, dent->d_name);
+    }else{
+      strcpy(path, dent->d_name);
+    }
+    space_escape(path);
+    if((r = mexec_scan_child(fd, path, sendhost, mode, c, gid))){
+      closedir(d);
+      return(r);
+    }
+  }
+  closedir(d);
   return(0);
 }
 
@@ -188,23 +191,23 @@ int mexec_scan_child(int fd, char *base, char *sendhost, int mode, mcomm *c, gid
   }
   /*----- exclude -----*/
   sprintf(path, "%s/%s", moption.real_dir, base);
-  if(!mfnmatch(path, c->exclude)){
-    if(!is_dir(base)){
-      return(mexec_scan_send(fd, base, sendhost, mode, gid));
-    }else{
-      /*----- exclude dir -----*/
-      strcat(path, "/");
-      if(mfnmatch(path, c->exclude)){
-        return(0);
-      }
-      /*----- scan dir -----*/
-      if(r = mexec_scan_dir(fd, base, sendhost, mode, c, 1, gid)){
-        return(r);
-      }
-      if(loop_flag && (mode != MAKUO_MEXEC_MD5)){
-        return(mexec_scan_send(fd, base, sendhost, mode, gid));
-      }
-    }
+  if(mfnmatch(path, c->exclude)){
+    return(0);
+  }
+  if(!is_dir(base)){
+    return(mexec_scan_send(fd, base, sendhost, mode, gid));
+  }
+  /*----- exclude dir -----*/
+  strcat(path, "/");
+  if(mfnmatch(path, c->exclude)){
+    return(0);
+  }
+  /*----- scan dir -----*/
+  if((r = mexec_scan_dir(fd, base, sendhost, mode, c, 1, gid))){
+    return(r);
+  }
+  if(loop_flag && (mode != MAKUO_MEXEC_MD5)){
+    return(mexec_scan_send(fd, base, sendhost, mode, gid));
   }
   return(0);
 }
@@ -222,12 +225,14 @@ int mexec_scan(mcomm *c, char *fn, mhost *h, int mode, gid_t gid)
     return(-1);
   }
 
-  base[0] = 0;
+  base[0]     = 0;
   sendhost[0] = 0;
-  if(h)
+  if(h){
     strcpy(sendhost, h->hostname);
-  if(fn)
+  }
+  if(fn){
     strcpy(base, fn);
+  }
 
   pid=fork();
   if(pid == -1){
@@ -254,6 +259,21 @@ int mexec_scan(mcomm *c, char *fn, mhost *h, int mode, gid_t gid)
   return(0);
 }
 
+int mexec_exclude_add(mcomm *c, char *pattern)
+{
+  c->exclude = exclude_add(c->exclude, pattern);
+  return(0);
+}
+
+int mexec_exclude_del(mcomm *c, excludeitem *e)
+{
+  excludeitem *d = exclude_del(e);
+  if(e == c->exclude){
+    c->exclude = d;
+  }
+  return(0);
+}
+
 int mexec_open(int l, mcomm *c, int n)
 {
   if(n){
@@ -276,7 +296,6 @@ int mexec_open(int l, mcomm *c, int n)
 int mexec_close(mcomm *c, int n)
 {
   mfile *m;
-
   if(c->fd[n] != -1){
     close(c->fd[n]);
     if(n){
@@ -293,7 +312,9 @@ int mexec_close(mcomm *c, int n)
   if(!n){
     c->authchk  = 0;
     c->logflag  = 0;
+    c->logover  = 0;
     c->loglevel = 0;
+    c->logfail  = 0;
     c->isalive  = 0;
     if(c->cpid){
       kill(c->cpid, SIGTERM);
@@ -337,6 +358,7 @@ int mexec_help(mcomm *c, int n)
   cprintf(0, c, "  alive on|off\n");
   cprintf(0, c, "  loglevel num (0-9)\n");
   cprintf(0, c, "  members\n");
+  cprintf(0, c, "  status\n");
   cprintf(0, c, "  help\n");
   return(0);
 }
@@ -381,11 +403,15 @@ int mexec_send(mcomm *c, int n, int sync)
     argv[i] = c->parse[n][i];
   }
   argv[i] = NULL;
+  
+  if(!strcmp("SunOS", moption.uts.sysname)){
+    optind = 1; /* solaris */
+  }else{
+    optind = 0; /* other */
+  }
+  opterr = 1;
 #ifdef HAVE_GETOPT_OPTRESET
-  optind   = 1;
   optreset = 1;
-#else
-  optind = 0;
 #endif
   while((i=getopt(c->argc[n], argv, "g:t:nr")) != -1){
     switch(i){
@@ -439,7 +465,7 @@ int mexec_send(mcomm *c, int n, int sync)
 
   if(fn){
     int len;
-    if(len = strlen(fn)){
+    if((len = strlen(fn))){
       if(fn[len - 1] == '/'){
         fn[len - 1] = 0;
       }
@@ -554,7 +580,6 @@ int mexec_check(mcomm *c, int n)
 {
   int e;
   int i;
-  ssize_t size;
   char *argv[9];
   char *fn = NULL;
   mfile *m = NULL;
@@ -565,11 +590,14 @@ int mexec_check(mcomm *c, int n)
   for(i=0;i<c->argc[n];i++)
     argv[i] = c->parse[n][i];
   argv[i] = NULL;
+  if(!strcmp("SunOS", moption.uts.sysname)){
+    optind = 1; /* solaris */
+  }else{
+    optind = 0; /* other */
+  }
+  opterr = 1;
 #ifdef HAVE_GETOPT_OPTRESET
-  optind   = 1;
   optreset = 1;
-#else
-  optind = 0;
 #endif
   while((i=getopt(c->argc[n], argv, "t:r")) != -1){
     switch(i){
@@ -659,7 +687,6 @@ int mexec_check(mcomm *c, int n)
 int mexec_dsync(mcomm *c, int n)
 {
   int i;
-  ssize_t size;
   char *argv[9];
   char *fn = NULL;
   mhost *t = NULL;
@@ -673,11 +700,14 @@ int mexec_dsync(mcomm *c, int n)
   for(i=0;i<c->argc[n];i++)
     argv[i] = c->parse[n][i];
   argv[i] = NULL;
+  if(!strcmp("SunOS", moption.uts.sysname)){
+    optind = 1; /* solaris */
+  }else{
+    optind = 0; /* other */
+  }
+  opterr = 1;
 #ifdef HAVE_GETOPT_OPTRESET
-  optind   = 1;
   optreset = 1;
-#else
-  optind = 0;
 #endif
   while((i=getopt(c->argc[n], argv, "t:nr")) != -1){
     switch(i){
@@ -708,7 +738,7 @@ int mexec_dsync(mcomm *c, int n)
 
   if(fn){
     int len;
-    if(len = strlen(fn)){
+    if((len = strlen(fn))){
       if(fn[len - 1] == '/'){
         fn[len - 1] = 0;
       }
@@ -734,7 +764,6 @@ int mexec_members(mcomm *c, int n)
   int counter = 0;
   int namelen = 0;
   int addrlen = 0;
-  int statcnt = 0;
   mhost *t;
   mhost **pt;
   char form[256];
@@ -771,6 +800,7 @@ int mexec_members(mcomm *c, int n)
 
   /* view */
 #ifdef MAKUO_DEBUG
+  int statcnt = 0;
   sprintf(form, "%%-%ds %%-%ds (Ver%%s) STATE_AREA(%%d/%%d)\n", namelen, addrlen);
   for(i=0;i<counter;i++){
     statcnt = 0;
@@ -811,21 +841,6 @@ int mexec_loglevel(mcomm *c, int n)
   return(0);
 }
 
-int mexec_exclude_add(mcomm *c, char *pattern)
-{
-  c->exclude = exclude_add(c->exclude, pattern);
-  return(0);
-}
-
-int mexec_exclude_del(mcomm *c, excludeitem *e)
-{
-  excludeitem *d = exclude_del(e);
-  if(e == c->exclude){
-    c->exclude = d;
-  }
-  return(0);
-}
-
 int mexec_exclude(mcomm *c, int n)
 {
   excludeitem *e;
@@ -883,16 +898,16 @@ int mexec_status(mcomm *c, int n)
   struct tm *t;
 
   /*----- pid -----*/
-  cprintf(0, c, "process: %d\n", getpid());
+  cprintf(0, c, "process : %d\n", getpid());
 
   /*----- version -----*/
-  cprintf(0,c,"version: %s\n", PACKAGE_VERSION);
+  cprintf(0,c,"version : %s\n", PACKAGE_VERSION);
 
   /*----- basedir -----*/
   if(moption.chroot){
-    cprintf(0, c, "chroot : %s/\n", moption.real_dir);
+    cprintf(0, c, "chroot  : %s/\n", moption.real_dir);
   }else{
-    cprintf(0, c, "basedir: %s/\n", moption.base_dir);
+    cprintf(0, c, "basedir : %s/\n", moption.base_dir);
   }
 
   /*----- mfalloc -----*/
@@ -906,7 +921,18 @@ int mexec_status(mcomm *c, int n)
   for(m=mfreeobj;m;m=m->next){
     count++;
   }
-  cprintf(0, c, "mfalloc: %d\n", count);
+  cprintf(0, c, "mfalloc : %d\n", count);
+
+  /*----- RCVBUF/SNDBUF -----*/
+  cprintf(0, c, "recvsize: %d\n", moption.recvsize);
+  cprintf(0, c, "sendsize: %d\n", moption.sendsize);
+  
+  /*----- send rate -----*/
+  if(moption.sendrate){
+    cprintf(0, c, "sendrate: %d/%d\n", 
+      view_rate        * 8 / 1024 / 1024, 
+      moption.sendrate * 8 / 1024 / 1024);
+  }
 
   /*----- command -----*/
   count = 0;
@@ -915,7 +941,7 @@ int mexec_status(mcomm *c, int n)
       count++;
     }
   }
-  cprintf(0, c, "command: %d\n", count);
+  cprintf(0, c, "command : %d\n", count);
   for(i=0;i<MAX_COMM;i++){
     if(moption.comm[i].working && (c != &(moption.comm[i]))){
       cprintf(0, c, "  %d> %s\n", i, moption.comm[i].cmdline[0]);
@@ -927,7 +953,7 @@ int mexec_status(mcomm *c, int n)
   for(m=mftop[MFSEND];m;m=m->next){
     count++;
   }
-  cprintf(0,c,"send op: %d\n", count);
+  cprintf(0,c,"send op : %d\n", count);
   for(m=mftop[MFSEND];m;m=m->next){
     uint32_t snow = m->seqnonow;
     uint32_t smax = m->seqnomax;
@@ -950,10 +976,10 @@ int mexec_status(mcomm *c, int n)
   count = 0;
   for(m=mftop[MFRECV];m;m=m->next)
     count++;
-  cprintf(0, c, "recv op: %d\n", count);
+  cprintf(0, c, "recv op : %d\n", count);
   for(m=mftop[MFRECV];m;m=m->next){
     t = localtime(&(m->lastrecv.tv_sec));
-    cprintf(0, c, "  %s %s %02d:%02d:%02d %s (%d/%d) mark=%d rid=%d\n",
+    cprintf(0, c, "  %s %s %02d:%02d:%02d %s (%d/%d) mark=%d rid=%d ip=%s\n",
       stropcode(&(m->mdata)), 
       strrstate(m->mdata.head.nstate), 
       t->tm_hour, t->tm_min, t->tm_sec, 
@@ -961,7 +987,8 @@ int mexec_status(mcomm *c, int n)
       m->recvcount,
       m->seqnomax, 
       m->markcount,
-      m->mdata.head.reqid); 
+      m->mdata.head.reqid,
+      inet_ntoa(m->addr.sin_addr)); 
   }
   return(0);
 }
@@ -1049,7 +1076,7 @@ int mexec_parse(mcomm *c, int n)
         break;
       strcpy(c->parse[n][j], p);
       if(j){
-        if(l = strlen(c->parse[n][j-1])){
+        if((l = strlen(c->parse[n][j-1]))){
           if(c->parse[n][j-1][l-1] == '\\'){
             c->parse[n][j-1][l-1] = 0;
             strcat(c->parse[n][j-1], " ");
