@@ -1,18 +1,15 @@
 /*
  * mexit.c
- * Copyright (C) 2008 KLab Inc.
+ * Copyright (C) 2008-2012 KLab Inc.
  */
 #include "makuosan.h"
 
 static void chexit()
 {
-  struct utsname uts;
+  int retrycnt = 256;
   char cwd[PATH_MAX];
   if(moption.chroot){
-    if(uname(&uts) == -1){
-      return;
-    }
-    if(strcmp("Linux", uts.sysname)){
+    if(strcmp("Linux", moption.uts.sysname)){
       return;
     }
     /*----- chroot exit(linux only) -----*/
@@ -22,9 +19,10 @@ static void chexit()
     rmdir(cwd);
     chdir("..");
     getcwd(cwd,PATH_MAX);
-    while(strcmp("/", cwd)){
+    while(retrycnt && strcmp("/", cwd) && strcmp("(unreachable)/", cwd)){
       chdir("..");
       getcwd(cwd,PATH_MAX);
+      retrycnt--;
     }
     chroot(".");
   }
