@@ -1,8 +1,11 @@
 /*
  * minit.c
- * Copyright (C) 2008 KLab Inc.
+ * Copyright (C) 2008-2012 KLab Inc.
  */
 #include "makuosan.h"
+#ifdef HAVE_SYS_PRCTL_H
+#include <sys/prctl.h>
+#endif
 
 static void version_print()
 {
@@ -14,25 +17,31 @@ static void usage()
   version_print();
   printf("(Multicasts All-Kinds of Updating Operation for Servers on Administered Network)\n\n");
   printf("usage: makuosan [OPTION]\n");
-  printf("  -d num   # loglevel(0-9)\n");
-  printf("  -u uid   # user\n");
-  printf("  -g gid   # group\n");
-  printf("  -G gid,..# groups\n");
-  printf("  -b dir   # base dir\n");
-  printf("  -p port  # port number       (default: 5000)\n");
-  printf("  -m addr  # multicast address (default: 224.0.0.108)\n");
-  printf("  -l addr  # listen address    (default: 127.0.0.1)\n");
-  printf("  -U path  # unix domain socket\n");
-  printf("  -k file  # key file (encrypt password)\n");
-  printf("  -K file  # key file (console password)\n");
-  printf("  -f num   # parallel send count(default: 5) \n");
-  printf("  -c       # chroot to base dir\n");
-  printf("  -n       # don't fork\n");
-  printf("  -r       # don't recv\n");
-  printf("  -s       # don't send\n");
-  printf("  -o       # don't listen (console off mode)\n");
-  printf("  -O       # owner match limitation mode\n");
-  printf("  -h       # help\n\n"); 
+  printf("  -d num       # loglevel(0-9)\n");
+  printf("  -u uid       # user\n");
+  printf("  -g gid       # group\n");
+  printf("  -G gid,..    # groups\n");
+  printf("  -b dir       # base dir\n");
+  printf("  -p port      # port number       (default: 5000)\n");
+  printf("  -m addr      # multicast address (default: 224.0.0.108)\n");
+  printf("  -i addr      # interface address (default: 0.0.0.0)\n");
+  printf("  -l addr      # listen address    (default: 127.0.0.1)\n");
+  printf("  -U path      # unix domain socket\n");
+  printf("  -k file      # key file (encrypt password)\n");
+  printf("  -K file      # key file (console password)\n");
+  printf("  -f num       # parallel send count(default: 5) \n");
+  printf("  -R num       # recv buffer size [bytes]\n");
+  printf("  -S num       # send buffer size [bytes]\n");
+  printf("  -T num       # traffic rate     [Mbps]\n");
+  printf("  -n           # don't fork\n");
+  printf("  -r           # don't recv\n");
+  printf("  -s           # don't send\n");
+  printf("  -o           # don't listen (console off mode)\n");
+  printf("  -O           # owner match limitation mode\n");
+  printf("  -c --chroot  # chroot to base dir\n");
+  printf("  -C --core    # enable core\n");
+  printf("  -V --version # version\n"); 
+  printf("  -h --help    # help\n\n"); 
   exit(0);
 }
 
@@ -46,15 +55,13 @@ static void signal_handler(int n)
     case SIGPIPE:
       break;
     case SIGUSR1:
-      if(moption.loglevel<9){
-        moption.loglevel++;
-        lprintf(0,"%s: loglevel=%d\n", __func__, moption.loglevel);
+      if(log_level<9){
+        log_level++;
       }
       break;
     case SIGUSR2:
-      if(moption.loglevel>0){
-        moption.loglevel--;
-        lprintf(0,"%s: loglevel=%d\n", __func__, moption.loglevel);
+      if(log_level>0){
+        log_level--;
       }
       break;
   }
@@ -67,6 +74,9 @@ static void minit_option_setdefault()
   moption.maddr.sin_family      = AF_INET;
   moption.maddr.sin_addr.s_addr = inet_addr(MAKUO_MCAST_ADDR);
   moption.maddr.sin_port        = htons(MAKUO_MCAST_PORT);
+  moption.iaddr.sin_family      = AF_INET;
+  moption.iaddr.sin_addr.s_addr = INADDR_ANY;
+  moption.iaddr.sin_port        = htons(MAKUO_MCAST_PORT);
   moption.laddr.sin_family      = AF_INET;
   moption.laddr.sin_addr.s_addr = inet_addr(MAKUO_LOCAL_ADDR);
   moption.laddr.sin_port        = htons(MAKUO_MCAST_PORT);
@@ -78,9 +88,14 @@ static void minit_option_setdefault()
   moption.dontfork              = 0;
   moption.cryptena              = 0;
   moption.comm_ena              = 1;
+  moption.core_ena              = 0;
+  moption.coresize              = 0;
   moption.commpass              = 0;
   moption.ownmatch              = 0;
   moption.parallel              = 5;
+  moption.recvsize              = 0;
+  moption.sendsize              = 0;
+  moption.sendrate              = 0;
   moption.chroot                = 0;
   moption.uid                   = geteuid();
   moption.gid                   = getegid();
@@ -91,6 +106,7 @@ static void minit_option_setdefault()
     moption.comm[i].fd[0] = -1;
     moption.comm[i].fd[1] = -1;
   }
+  uname(&moption.uts);
 }
 
 static int minit_option_setuid(char *name)
@@ -99,7 +115,7 @@ static int minit_option_setuid(char *name)
   if(*name >= '0' && *name <= '9'){
     moption.uid = atoi(name);
   }else{
-    if(pw = getpwnam(name)){
+    if((pw = getpwnam(name))){
       moption.uid = pw->pw_uid;
       moption.gid = pw->pw_gid;
     }else{
@@ -116,7 +132,7 @@ static int minit_option_setgid(char *name)
   if(*name >= '0' && *name <='9'){
     moption.gid = atoi(name);
   }else{
-    if(gr = getgrnam(name)){
+    if((gr = getgrnam(name))){
       moption.gid = gr->gr_gid;
     }else{
       lprintf(0,"[error] %s: not found group %s\n", __func__, name);
@@ -168,7 +184,7 @@ static int minit_option_setgids(char *name)
   while(p){
     if(*p >= '0' && *p <= '9'){
       gid = atoi(p);
-      if(g = getgrgid(gid)){
+      if((g = getgrgid(gid))){
         moption.gids[num] = gid;
         strcpy(moption.grnames[num], g->gr_name);
       }else{
@@ -176,7 +192,7 @@ static int minit_option_setgids(char *name)
         return(1);
       }
     }else{
-      if(g = getgrnam(p)){
+      if((g = getgrnam(p))){
         moption.gids[num] = g->gr_gid;
         strcpy(moption.grnames[num], p);
       }else{
@@ -193,34 +209,42 @@ static int minit_option_setgids(char *name)
 static void minit_option_getenv()
 {
   char *env;
-  struct passwd *pw;
-  struct group  *gr;
 
-  if(env=getenv("MAKUOSAN_BASE")){
-    realpath(env, moption.base_dir);
+  if((env=getenv("MAKUOSAN_BASE"))){
+    if(*env){
+      realpath(env, moption.base_dir);
+    }
   }
-  if(env=getenv("MAKUOSAN_PORT")){
-    moption.maddr.sin_port = htons(atoi(env));
-    moption.laddr.sin_port = htons(atoi(env));
+  if((env=getenv("MAKUOSAN_PORT"))){
+    if(*env && atoi(env)){
+      moption.maddr.sin_port = htons(atoi(env));
+      moption.laddr.sin_port = htons(atoi(env));
+    }
   }
-  if(env=getenv("MAKUOSAN_USER")){
-    if(minit_option_setuid(env)){
+  if((env=getenv("MAKUOSAN_USER"))){
+    if(*env && minit_option_setuid(env)){
       exit(1);
     }
   }
-  if(env=getenv("MAKUOSAN_GROUP")){
-    if(minit_option_setgid(env)){
+  if((env=getenv("MAKUOSAN_GROUP"))){
+    if(*env && minit_option_setgid(env)){
       exit(1);
     }
   }
-  if(env=getenv("MAKUOSAN_GROUPS")){
-    if(minit_option_setgids(env)){
+  if((env=getenv("MAKUOSAN_GROUPS"))){
+    if(*env && minit_option_setgids(env)){
       exit(1);
     }
   }
-  if(env=getenv("MAKUOSAN_SOCK")){
+  if((env=getenv("MAKUOSAN_SOCK"))){
     strcpy(moption.uaddr.sun_path, env);
   }
+  if((env=getenv("MAKUOSAN_RCVBUF"))){
+    moption.recvsize = atoi(env);
+  }
+  if((env=getenv("MAKUOSAN_SNDBUF"))){
+    moption.sendsize = atoi(env);
+  }
 }
 
 static void minit_signal()
@@ -291,8 +315,15 @@ static void minit_password(char *filename, int n)
 static void minit_getopt(int argc, char *argv[])
 {
   int r;
+  struct option opt[]={
+    {"chroot",  0, NULL, 'c'},
+    {"core",    0, NULL, 'C'},
+    {"help",    0, NULL, 'h'},
+    {"version", 0, NULL, 'V'},
+    {0, 0, 0, 0}
+  };
 
-  while((r=getopt(argc, argv, "f:u:g:G:d:b:p:m:l:U:k:K:VhnsroOc")) != -1){
+  while((r=getopt_long(argc, argv, "T:R:S:f:u:g:G:d:b:p:m:i:l:U:k:K:VhnsroOcC", opt, NULL)) != -1){
     switch(r){
       case 'V':
         version_print();
@@ -302,6 +333,18 @@ static void minit_getopt(int argc, char *argv[])
         usage();
         exit(0);
 
+      case 'T':
+        moption.sendrate = atoi(optarg) * 1024 * 1024 / 8;
+        break;
+
+      case 'R':
+        moption.recvsize = atoi(optarg);
+        break;
+
+      case 'S':
+        moption.sendsize = atoi(optarg);
+        break;
+
       case 'f':
         moption.parallel = atoi(optarg);
         if(moption.parallel < 1){
@@ -328,6 +371,10 @@ static void minit_getopt(int argc, char *argv[])
         moption.comm_ena = 0;
         break;
 
+      case 'C':
+        moption.core_ena = 1;
+        break;
+
       case 'c':
         moption.chroot = 1;
         break;
@@ -362,6 +409,10 @@ static void minit_getopt(int argc, char *argv[])
         moption.maddr.sin_addr.s_addr = inet_addr(optarg);
         break;
 
+      case 'i':
+        moption.iaddr.sin_addr.s_addr = inet_addr(optarg);
+        break;
+
       case 'l':
         moption.laddr.sin_addr.s_addr = inet_addr(optarg);
         break;
@@ -388,11 +439,12 @@ static void minit_getopt(int argc, char *argv[])
       case 'O':
         moption.ownmatch = 1;
         break;
-
+  
       case '?':
         exit(1);
     }
   }
+  log_level = moption.loglevel;
 }
 
 static void minit_syslog()
@@ -403,13 +455,13 @@ static void minit_syslog()
 static void minit_socket()
 {
   int  s;
-  int  reuse =  1;
-  char lpen  =  0;
-  char mttl  =  1;
+  char lpen  = 0;
+  char mttl  = 1;
+  socklen_t slen;
   struct ip_mreq mg;
   struct sockaddr_in addr;
   mg.imr_multiaddr.s_addr = moption.maddr.sin_addr.s_addr;
-  mg.imr_interface.s_addr = INADDR_ANY;
+  mg.imr_interface.s_addr = moption.iaddr.sin_addr.s_addr;
   addr.sin_family         = AF_INET;
   addr.sin_port           = moption.maddr.sin_port; 
   addr.sin_addr.s_addr    = INADDR_ANY;
@@ -419,6 +471,30 @@ static void minit_socket()
     lprintf(0, "%s: can't create multicast socket\n", __func__);
     exit(1);
   }
+  if(moption.recvsize){
+    if(setsockopt(s, SOL_SOCKET, SO_RCVBUF, (void *)&(moption.recvsize), sizeof(moption.recvsize)) == -1){
+      lprintf(0, "%s: setsockopt SO_RCVBUF error\n", __func__);
+      exit(1);
+    }
+  }
+  slen=sizeof(moption.recvsize);
+  if(getsockopt(s, SOL_SOCKET, SO_RCVBUF, (void *)&(moption.recvsize), &slen) == -1){
+    lprintf(0, "%s: getsockopt SO_RCVBUF error\n", __func__);
+    exit(1);
+  }
+  moption.recvsize /= 2;
+  if(moption.sendsize){
+    if(setsockopt(s, SOL_SOCKET, SO_SNDBUF, (void *)&(moption.sendsize), sizeof(moption.sendsize)) == -1){
+      lprintf(0, "%s: setsockopt SO_SNDBUF error\n", __func__);
+      exit(1);
+    }
+  }
+  slen=sizeof(moption.sendsize);
+  if(getsockopt(s, SOL_SOCKET, SO_SNDBUF, (void *)&(moption.sendsize), &slen) == -1){
+    lprintf(0, "%s: getsockopt SO_SNDBUF error\n", __func__);
+    exit(1);
+  }
+  moption.sendsize /= 2;
   if(fcntl(s, F_SETFL , O_NONBLOCK)){
     lprintf(0, "%s: fcntl error\n", __func__);
     exit(1);
@@ -508,15 +584,12 @@ static void minit_chdir()
 
 static void minit_chroot()
 {
-  time_t ttime;
-  struct tm *t;
   char tz[256];
   if(moption.chroot){
-    time(&ttime);
     tzset();
-    t = localtime(&ttime);
-    sprintf(tz, "%s%+ld",   t->tm_zone, -(t->tm_gmtoff/3600));
+    sprintf(tz, "%s%+ld", tzname[0], timezone/3600);
     setenv("TZ", tz, 0);
+    tzset();
     if(chroot(moption.base_dir) == -1){
       fprintf(stderr, "%s: can't chroot %s\n", __func__, moption.base_dir);
       exit(1);
@@ -529,10 +602,10 @@ static void minit_getguid()
 {
   struct passwd *pw;
   struct group  *gr;
-  if(pw = getpwuid(moption.uid)){
+  if((pw = getpwuid(moption.uid))){
     strcpy(moption.user_name, pw->pw_name);
   }
-  if(gr = getgrgid(moption.gid)){
+  if((gr = getgrgid(moption.gid))){
     strcpy(moption.group_name,gr->gr_name);
   }
 }
@@ -540,8 +613,6 @@ static void minit_getguid()
 static void minit_setguid()
 {
   size_t num;
-  struct passwd *pw;
-  struct group  *gr;
   if(set_guid(moption.uid, moption.gid, moption.gidn, moption.gids) == -1){
     fprintf(stderr, "%s: can't setguid %d:%d", __func__, moption.uid, moption.gid);
     if(moption.gidn){
@@ -554,11 +625,43 @@ static void minit_setguid()
   }
 }
 
+static void minit_enable_core()
+{
+  struct rlimit r = {0, 0};
+  if(getrlimit(RLIMIT_CORE, &r) == -1){
+    fprintf(stderr, "%s: getrlimit error: %s\n", __func__, strerror(errno));
+    exit(1);
+  }
+  moption.coresize = (int)(r.rlim_cur);
+  if(!moption.core_ena){
+    return;
+  }
+#ifdef HAVE_SYS_PRCTL_H
+  if(prctl(PR_SET_DUMPABLE, 1) == -1){
+    fprintf(stderr, "%s: prctl error: %s\n", __func__, strerror(errno));
+  }
+#endif
+  r.rlim_cur = r.rlim_max;
+  if(setrlimit(RLIMIT_CORE, &r) == -1){
+    fprintf(stderr, "%s: setrlimit error: %s\n", __func__, strerror(errno));
+    exit(1);
+  }
+  r.rlim_cur = 0;
+  r.rlim_max = 0;
+  if(getrlimit(RLIMIT_CORE, &r) == -1){
+    fprintf(stderr, "%s: getrlimit error: %s\n", __func__, strerror(errno));
+    exit(1);
+  }
+  moption.coresize = (int)(r.rlim_cur);
+}
+
 static void minit_daemonize()
 {
   int pid;
-  if(moption.dontfork)
+  if(moption.dontfork){
+    lprintf(0, "pid       : %d\n", getpid());
     return;
+  }
 
   pid = fork();
   if(pid == -1){
@@ -573,8 +676,10 @@ static void minit_daemonize()
     fprintf(stderr, "%s: can't fork()\n", __func__);
     exit(1); 
   }
-  if(pid)
+  if(pid){
+    lprintf(0, "pid       : %d\n",pid);
     _exit(0);
+  }
 
   /*----- daemon process -----*/
   close(2);
@@ -590,13 +695,20 @@ static void minit_bootlog()
   int i;
 
   lprintf(0, "makuosan version %s\n", PACKAGE_VERSION);
+  lprintf(0, "sysname   : %s\n", moption.uts.sysname);
   lprintf(0, "loglevel  : %d\n", moption.loglevel);
+  if(moption.coresize == (int)RLIM_INFINITY){
+    lprintf(0, "coresize  : unlimited\n");
+  }else{
+    lprintf(0, "coresize  : %d\n", moption.coresize);
+  }
   if(moption.chroot){
     lprintf(0, "chroot    : %s\n", moption.real_dir);
   }else{
     lprintf(0, "base dir  : %s\n", moption.base_dir);
   }
   lprintf(0, "multicast : %s\n", inet_ntoa(moption.maddr.sin_addr));
+  lprintf(0, "interface : %s\n", inet_ntoa(moption.iaddr.sin_addr));
   lprintf(0, "port      : %d\n", ntohs(moption.maddr.sin_port));
   lprintf(0, "uid       : %d(%s)\n", moption.uid, moption.user_name);
   lprintf(0, "gid       : %d(%s)"  , moption.gid, moption.group_name);
@@ -606,6 +718,8 @@ static void minit_bootlog()
     }
   }
   lprintf(0, "\n");
+  lprintf(0, "rcvbuf    : %d\n", moption.recvsize);
+  lprintf(0, "sndbuf    : %d\n", moption.sendsize);
   lprintf(0, "parallel  : %d\n", moption.parallel);
   lprintf(0, "don't recv: %s\n", yesno(moption.dontrecv));
   lprintf(0, "don't send: %s\n", yesno(moption.dontsend));
@@ -614,6 +728,9 @@ static void minit_bootlog()
   lprintf(0, "console   : %s\n", yesno(moption.comm_ena));
   lprintf(0, "passwoed  : %s\n", yesno(moption.commpass));
   lprintf(0, "ownermatch: %s\n", yesno(moption.ownmatch));
+  if(moption.sendrate){
+    lprintf(0, "rate      : %d[Mbps]\n", moption.sendrate * 8 / 1024 / 1024);
+  }
   if(moption.comm_ena){
     if(moption.uaddr.sun_path[0]){
       lprintf(0,"listen    : %s\n", moption.uaddr.sun_path);
@@ -643,6 +760,7 @@ void minit(int argc, char *argv[])
   minit_getguid();           /*                                    */
   minit_chroot();            /*                                    */
   minit_setguid();           /*                                    */
+  minit_enable_core();       /* コアサイズのリミットを解除         */
   minit_bootlog();           /* ブートメッセージを出力する         */
   minit_daemonize();         /*                                    */
 }
