@@ -1,8 +1,9 @@
 /*
  * mrecv.c
- * Copyright (C) 2008 KLab Inc.
+ * Copyright (C) 2008-2012 KLab Inc.
  */
 #include "makuosan.h"
+#include <openssl/md5.h>
 
 static void mrecv_req(mdata *data, struct sockaddr_in *addr);
 static void mrecv_ack(mdata *data, struct sockaddr_in *addr);
@@ -30,7 +31,7 @@ static mfile *mrecv_mfdel(mfile *m)
     m->link->link = NULL;
     m->link = NULL;
   }
-  while(m->mark = delmark(m->mark));
+  while((m->mark = delmark(m->mark)));
   mfdel(m);
   return(r);
 }
@@ -97,8 +98,8 @@ static int mrecv_packet(int s, mdata *data, struct sockaddr_in *addr)
     data->head.flags  = ntohs(data->head.flags);
     data->head.reqid  = ntohl(data->head.reqid);
     data->head.seqno  = ntohl(data->head.seqno);
-    data->head.maddr  = ntohl(data->head.maddr);
-    data->head.mport  = ntohs(data->head.mport);
+    data->head.maddr  = data->head.maddr;
+    data->head.mport  = data->head.mport;
     data->head.error  = ntohl(data->head.error);
     if(data->head.maddr != moption.maddr.sin_addr.s_addr){
       continue; /* other group packet */
@@ -297,6 +298,11 @@ static void mrecv_ack_del(mdata *data, struct sockaddr_in *addr)
   if(mrecv_ack_search(&t, &m, data, addr)){
     return;
   }
+  if(m->mdata.head.nstate == MAKUO_SENDSTATE_CLOSE){
+    if(data->head.nstate != MAKUO_RECVSTATE_CLOSE){
+      return;
+    }
+  }
   if(!set_hoststate(t, m, data->head.nstate)){
     lprintf(0, "%s: not allocate state area\n", __func__);
     return;
@@ -318,13 +324,13 @@ static void mrecv_ack_del(mdata *data, struct sockaddr_in *addr)
 
         err = 0;
         if(m->dryrun){
-          lprintf(1, "%s: (dryrun) delete %s\n", __func__, path);
+          lprintf(1, "%s: (dryrun) delete %s rid=%d\n", __func__, path, m->mdata.head.reqid);
         }else{
           if(!mremove(NULL,path)){
-            lprintf(1, "%s: delete %s\n", __func__, path);
+            lprintf(1, "%s: delete %s rid=%d\n", __func__, path, m->mdata.head.reqid);
           }else{
             err = errno;
-            lprintf(0, "%s: delete error %s (%s)\n", __func__, path, strerror(errno));
+            lprintf(0, "%s: delete error %s (%s) rid=%d\n", __func__, path, strerror(errno), m->mdata.head.reqid);
           }
         }
         data_safeset16(&(m->mdata), len + sizeof(uint32_t));
@@ -417,7 +423,6 @@ static void mrecv_req_send_break(mfile *m, mdata *r)
 static void mrecv_req_send_stat(mfile *m, mdata *r)
 {
   struct stat fs;
-  struct utimbuf mftime;
 
   if(moption.dontrecv){
     m->mdata.head.nstate = MAKUO_RECVSTATE_READONLY;
@@ -456,6 +461,9 @@ static void mrecv_req_send_open(mfile *m, mdata *r)
   char tpath[PATH_MAX];
 
   if(m->mdata.head.nstate != MAKUO_RECVSTATE_UPDATE){
+    if(m->mdata.head.ostate == MAKUO_RECVSTATE_UPDATE){
+      msend(mkack(&(m->mdata), &(m->addr), m->mdata.head.nstate));
+    }
     return;
   }
 
@@ -802,7 +810,7 @@ static mfile *mrecv_req_send_create(mdata *data, struct sockaddr_in *addr)
   uint32_t  hdev;
   uint64_t  rdev;
 
-  if(m = mrecv_req_search(data, addr)){
+  if((m = mrecv_req_search(data, addr))){
     return(m);
   }
 
@@ -862,7 +870,7 @@ static mfile *mrecv_req_send_create(mdata *data, struct sockaddr_in *addr)
 static void mrecv_req_send(mdata *data, struct sockaddr_in *addr)
 {
   mfile *m;
-  if(m = mrecv_req_send_create(data, addr)){
+  if((m = mrecv_req_send_create(data, addr))){
     mtimeget(&(m->lastrecv));
     mrecv_req_send_next(m, data);
   }else{
@@ -1009,6 +1017,10 @@ static int dsync_scan(int fd, char *base, int recurs, excludeitem *e)
   if(!loop_flag){
     return(1);
   }
+  /*----- read only -----*/
+  if(moption.dontrecv){
+    return(0);
+  }
   /*----- exclude -----*/
   if(isexclude(base, e, 0)){
     return(0);
@@ -1029,8 +1041,8 @@ static int dsync_scan(int fd, char *base, int recurs, excludeitem *e)
   }
 
   /*----- dir scan -----*/
-  if(d = opendir(base)){
-    while(dent=readdir(d)){
+  if((d = opendir(base))){
+    while((dent=readdir(d))){
       if(!loop_flag){
         break;
       }
@@ -1042,13 +1054,13 @@ static int dsync_scan(int fd, char *base, int recurs, excludeitem *e)
       }
       sprintf(path, "%s/%s", base, dent->d_name);
       if(recurs){
-        if(r = dsync_scan(fd, path, recurs, e)){
+        if((r = dsync_scan(fd, path, recurs, e))){
           closedir(d);
           return(r);
         }
       }else{
         len = strlen(path);
-        if(r = dsync_write(fd, path, MAKUO_SENDSTATE_STAT, len, st.st_mode)){
+        if((r = dsync_write(fd, path, MAKUO_SENDSTATE_STAT, len, st.st_mode))){
           closedir(d);
           return(r);
         }
@@ -1145,7 +1157,7 @@ static void mrecv_req_dsync_data(mfile *m, mdata *data, struct sockaddr_in *addr
     d->pid  = pid;
     d->pipe = p[0];
     close(p[1]); /* write close */
-    while(d->exclude = exclude_del(d->exclude));
+    while((d->exclude = exclude_del(d->exclude)));
   }else{
     /* child */
     close(p[0]); /* read close */
@@ -1211,8 +1223,9 @@ static void mrecv_req_del_open(mdata *data, struct sockaddr_in *addr)
 {
   uint16_t len;
   uint32_t mod;
-  mhost *t = member_get(&(addr->sin_addr));
   mfile *a = mkack(data, addr, MAKUO_RECVSTATE_OPEN);
+  mfile *m = mrecv_req_search(data, addr);
+  mhost *t = member_get(&(addr->sin_addr));
   char path[PATH_MAX];
 
   if(!a){
@@ -1225,13 +1238,35 @@ static void mrecv_req_del_open(mdata *data, struct sockaddr_in *addr)
     len -= sizeof(uint32_t);
     data_safeget(data, path, len);
     path[len] =  0;
+#ifdef MAKUO_DEBUG
+    lprintf(9, "%s: %s", __func__, path);
+#endif
     if(lstat(path, &(a->fs)) == -1 && errno == ENOENT){
+#ifdef MAKUO_DEBUG
+      lprintf(9, " [DELETE]");
+#endif
       data_safeset16(&(a->mdata), len + sizeof(uint32_t));
       data_safeset32(&(a->mdata), 0);
       data_safeset(&(a->mdata), path, len);
     }
+#ifdef MAKUO_DEBUG
+    lprintf(9, "\n");
+#endif
   }
   msend(a);
+
+  if(m){
+    return;
+  }
+  m = mfadd(MFRECV);
+  m->mdata.head.opcode = data->head.opcode;
+  m->mdata.head.reqid  = data->head.reqid;
+  m->mdata.head.nstate = MAKUO_RECVSTATE_OPEN;
+  memcpy(&(m->addr), addr, sizeof(m->addr));
+  mtimeget(&(m->lastrecv));
+  if(data->head.flags & MAKUO_FLAG_DRYRUN){
+    m->dryrun = 1;
+  }
 }
 
 static void mrecv_req_del_data_report(mfile *m, mcomm *c, uint32_t err, char *hn, char *path)
@@ -1262,20 +1297,16 @@ static void mrecv_req_del_data(mdata *data, struct sockaddr_in *addr)
   char path[PATH_MAX];
 
   msend(mkack(data, addr, MAKUO_RECVSTATE_OPEN));
-  if(m){
+  if(!m){
+    return;
+  }
+  mtimeget(&(m->lastrecv));
+  if(m->mdata.head.nstate != MAKUO_RECVSTATE_OPEN){
     return;
   }
   if(t){
     hn = t->hostname;
   }
-  m = mfadd(MFRECV);
-  m->mdata.head.opcode = data->head.opcode;
-  m->mdata.head.reqid  = data->head.reqid;
-  m->mdata.head.nstate = MAKUO_RECVSTATE_OPEN;
-  memcpy(&(m->addr), addr, sizeof(m->addr));
-  if(data->head.flags & MAKUO_FLAG_DRYRUN){
-    m->dryrun = 1;
-  }
   for(a=mftop[MFSEND];a;a=a->next){
     if((a->mdata.head.reqid == data->head.seqno) && (a->comm != NULL)){
       c = a->comm;
@@ -1290,6 +1321,8 @@ static void mrecv_req_del_data(mdata *data, struct sockaddr_in *addr)
     path[len] =  0;
     mrecv_req_del_data_report(m, c, err, hn, path);
   }
+  m->mdata.head.ostate = m->mdata.head.nstate;
+  m->mdata.head.nstate = MAKUO_RECVSTATE_CLOSE;
 }
 
 static void mrecv_req_del_close(mdata *data, struct sockaddr_in *addr)
@@ -1382,7 +1415,7 @@ void mrecv_gc()
 void mrecv_clean()
 {
   mfile *m = mftop[MFRECV];
-  while(m=mrecv_mfdel(m));
+  while((m = mrecv_mfdel(m)));
 }
 
 int mrecv()
@@ -1396,7 +1429,7 @@ int mrecv()
   if(mrecv_packet(moption.mcsocket, &data, &addr) == -1){
     return(0);
   }
-  if(t = member_get(&addr.sin_addr)){
+  if((t = member_get(&addr.sin_addr))){
     mtimeget(&(t->lastrecv));
   }
   if(data.head.flags & MAKUO_FLAG_ACK){
