@@ -1,6 +1,6 @@
 /*
  * msend.c
- * Copyright (C) 2008 KLab Inc. 
+ * Copyright (C) 2008-2012 KLab Inc. 
  */
 #include "makuosan.h"
 
@@ -30,7 +30,7 @@ static mfile *msend_mfdel(mfile *m)
     m->link->link = NULL;
     m->link = NULL;
   }
-  while(m->mark = delmark(m->mark));
+  while((m->mark = delmark(m->mark)));
   clr_hoststate(m);
   mfdel(m);
   return(r);
@@ -59,7 +59,24 @@ static int msend_encrypt(mdata *data)
 static int msend_readywait()
 {
   fd_set fds;
+  time_t tm;
   struct timeval tv;
+
+  if(moption.sendrate){
+    tm = time(NULL);
+    while(tm == send_time){
+      if(moption.sendrate > send_rate){
+        break;
+      }
+      usleep(1000);
+      tm = time(NULL);
+    }
+    if(tm != send_time){
+      view_rate = send_rate;
+      send_rate =  0;
+      send_time = tm;
+    }
+  }
   while(!moption.sendready){
     FD_ZERO(&fds);
     FD_SET(moption.mcsocket, &fds);
@@ -91,8 +108,8 @@ static int msend_packet(int s, mdata *data, struct sockaddr_in *addr)
   senddata.head.flags  = htons(senddata.head.flags);
   senddata.head.reqid  = htonl(senddata.head.reqid);
   senddata.head.seqno  = htonl(senddata.head.seqno);
-  senddata.head.maddr  = htonl(senddata.head.maddr);
-  senddata.head.mport  = htons(senddata.head.mport);
+  senddata.head.maddr  = senddata.head.maddr;
+  senddata.head.mport  = senddata.head.mport;
   senddata.head.error  = htonl(senddata.head.error);
   szdata += sizeof(mhead);
  
@@ -100,6 +117,7 @@ static int msend_packet(int s, mdata *data, struct sockaddr_in *addr)
     moption.sendready = 0;
     r = sendto(s, &senddata, szdata, 0, (struct sockaddr*)addr, sizeof(struct sockaddr_in));
     if(r == szdata){
+      send_rate += r;
       return(0); /* success */
     }
     if(r == -1){
@@ -133,7 +151,6 @@ static int msend_packet(int s, mdata *data, struct sockaddr_in *addr)
 /* retry */
 static int msend_retry(mfile *m)
 {
-  uint32_t w;
   uint8_t *r;
   mhost   *t;
 
@@ -202,8 +219,8 @@ static void msend_ack_send(int s, mfile *m)
 static void msend_ack_md5(int s, mfile *m)
 {
   int r;
-  char hash[16];
-  char buff[8192];
+  unsigned char hash[16];
+  unsigned char buff[8192];
   mfile *d = m->link;
   if(!d){
     msend_shot(s, m);
@@ -366,7 +383,7 @@ static void msend_req_send_stat_delete_report(mfile *m)
         continue;
       }
     }
-    if(r = get_hoststate(t, m)){
+    if((r = get_hoststate(t, m))){
       if(*r == MAKUO_RECVSTATE_DELETEOK){
         cprintf(1, m->comm, "%sdelete %s:%s\n", dryrun, t->hostname, m->fn);
         lprintf(1, "%sdelete %s:%s\n", dryrun, t->hostname, m->fn);
@@ -398,7 +415,7 @@ static void msend_req_send_stat_update_report(mfile *m)
         continue;
       }
     }
-    if(r = get_hoststate(t, m)){
+    if((r = get_hoststate(t, m))){
       if(*r == MAKUO_RECVSTATE_UPDATE){
         cprintf(1, m->comm, "%supdate %s:%s\r\n", dryrun, t->hostname, m->fn);
         lprintf(1, "%supdate %s:%s\n", dryrun, t->hostname, m->fn);
@@ -517,18 +534,21 @@ static void msend_req_send_markdata(int s, mfile *m)
   if(r>0){
     m->mdata.head.szdata = r;
     msend_packet(s, &(m->mdata), &(m->addr));
-  }else{
-    if(!r){
-      lprintf(0, "%s: read eof? seqno=%d\n", __func__, m->mdata.head.seqno);
-    }else{
-      lprintf(0, "[error] %s: can't read (%s) seqno=%d %s\n",   __func__, strerror(errno), m->mdata.head.seqno, m->fn);
-      cprintf(0, m->comm, "error: can't read (%s) seqno=%d %s\n", strerror(errno), m->mdata.head.seqno, m->fn);
+    if(!m->mark){
+      m->initstate = 1;
+      m->mdata.head.nstate = MAKUO_SENDSTATE_MARK;
     }
+    return;
   }
-  if(!m->mark){
-    m->initstate = 1;
-    m->mdata.head.nstate = MAKUO_SENDSTATE_MARK;
+  if(!r){
+    lprintf(0, "[error] %s: read eof? seqno=%d %s\n", __func__, m->mdata.head.seqno, m->fn);
+    cprintf(0, m->comm, "error: read eof? seqno=%d %s\n", m->mdata.head.seqno, m->fn);
+  }else{
+    lprintf(0, "[error] %s: can't read (%s) seqno=%d %s\n",   __func__, strerror(errno), m->mdata.head.seqno, m->fn);
+    cprintf(0, m->comm, "error: can't read (%s) seqno=%d %s\n", strerror(errno), m->mdata.head.seqno, m->fn);
   }
+  m->mdata.head.nstate = MAKUO_SENDSTATE_BREAK;
+  m->initstate = 1;
 }
 
 static void msend_req_send_filedata(int s, mfile *m)
@@ -552,6 +572,8 @@ static void msend_req_send_filedata(int s, mfile *m)
       /* err */
       lprintf(0, "[error] %s: can't read (%s) seqno=%d %s\n",   __func__, strerror(errno), m->mdata.head.seqno, m->fn);
       cprintf(0, m->comm, "error: can't read (%s) seqno=%d %s\n", strerror(errno), m->mdata.head.seqno, m->fn);
+      m->mdata.head.nstate = MAKUO_SENDSTATE_BREAK;
+      m->initstate = 1;
     }else{
       /* eof */
       lprintf(9, "%s: block send count=%d %s\n", __func__, m->mdata.head.seqno, m->fn);
@@ -998,7 +1020,7 @@ static void msend_req_del_stat_read(int s, mfile *m)
   }
 
   while(1){
-    if(r = atomic_read(m->pipe, &(m->len), sizeof(m->len), 1)){
+    if((r = atomic_read(m->pipe, &(m->len), sizeof(m->len), 1))){
       if(r == -1){
         if(errno == EAGAIN){
           return;
@@ -1065,20 +1087,20 @@ static void msend_req_del_stat(int s, mfile *m)
 {
   if(m->pipe != -1){
     msend_req_del_stat_read(s, m);
+    return;
+  }
+  if(msend_req_del_stat_waitcheck(s, m)){
+    m->sendwait = 1;
+    return;
+  }
+  if(m->link){
+    msend(mkack(&(m->link->mdata), &(m->link->addr), MAKUO_RECVSTATE_CLOSE)); /* send ack for dsync */
+  }
+  if(waitpid(m->pid, NULL, WNOHANG) != m->pid){
+    m->sendwait = 1;
   }else{
-    if(msend_req_del_stat_waitcheck(s, m)){
-      m->sendwait = 1;
-    }else{
-      if(m->link){
-        msend(mkack(&(m->link->mdata), &(m->link->addr), MAKUO_RECVSTATE_CLOSE));
-      }
-      if(waitpid(m->pid, NULL, WNOHANG) != m->pid){
-        m->sendwait = 1;
-      }else{
-        m->pid = 0;
-        msend_mfdel(m);
-      }
-    }
+    m->pid = 0;
+    msend_mfdel(m);
   }
 }
 
@@ -1088,6 +1110,7 @@ static void msend_req_del_break(int s, mfile *m)
   for(d=mftop[MFSEND];d;d=d->next){
     if(d->link == m){
       if(d->mdata.head.nstate == MAKUO_SENDSTATE_WAIT){
+        d->link = NULL;
         msend_mfdel(d);
         break;
       }
@@ -1230,9 +1253,10 @@ void msend(mfile *m)
     }
   }
 }
+
 void msend_clean()
 {
   mfile *m = mftop[MFSEND];
-  while(m=msend_mfdel(m));
+  while((m = msend_mfdel(m)));
 }
 
